@@ -1,14 +1,38 @@
 import logging
 
+from gettext import translation
+
+from babel import parse_locale
+
 import formencode
+
 import pylons
 import pylons.i18n
 from pylons.i18n import add_fallback, LanguageError, get_lang
-from pylons import config, session
+from pylons.configuration import config
+from pylons import session
+
 
 log = logging.getLogger(__name__)
 
 
+def sanitize_language_code(lang):
+    """Sanitize the language code if the spelling is slightly wrong.
+
+    For instance, 'pt-br' and 'pt_br' should be interpreted as 'pt_BR'.
+
+    """
+    try:
+        lang = '_'.join(filter(None, parse_locale(lang)[:2]))
+    except ValueError:
+        if '-' in lang:
+            try:
+                lang = '_'.join(filter(None, parse_locale(lang, sep='-')[:2]))
+            except ValueError:
+                pass
+    return lang
+
+
 def setup_i18n():
     """Set languages from the request header and the session.
 
@@ -16,6 +40,7 @@ def setup_i18n():
 
     Automatically called by tg controllers to setup i18n.
     Should only be manually called if you override controllers function.
+
     """
     if pylons.session:
         # If session is available, we try to see if there are languages set
@@ -28,7 +53,8 @@ def setup_i18n():
             languages = []
     else:
         languages = []
-    languages.extend(pylons.request.accept_language.best_matches())
+    languages.extend(map(sanitize_language_code,
+        pylons.request.accept_language.best_matches()))
     set_temporary_lang(languages)
 
 
@@ -38,19 +64,21 @@ def set_temporary_lang(languages):
 
     languages should be a string or a list of strings.
     First lang will be used as main lang, others as fallbacks.
+
     """
+    # the logging to the screen was removed because
+    # the printing to the screen for every problem causes serious slow down.
+
     try:
         pylons.i18n.set_lang(languages)
     except LanguageError:
-        log.info("Language %s: not supported", languages)
-    else:
-        log.info("Set request language to %s", languages)
+        pass
+        #log.warn("Language %s: not supported", languages)
     try:
         set_formencode_translation(languages)
     except LanguageError:
-        log.info("Language %s: not supported by FormEncode", languages)
-    else:
-        log.info("Set request language for FormEncode to %s", languages)
+        pass
+        #log.warn("Language %s: not supported by FormEncode", languages)
 
 
 def set_lang(languages, **kwargs):
@@ -59,6 +87,7 @@ def set_lang(languages, **kwargs):
 
     languages should be a string or a list of strings.
     First lang will be used as main lang, others as fallbacks.
+
     """
     set_temporary_lang(languages)
 
@@ -67,12 +96,13 @@ def set_lang(languages, **kwargs):
         session.save()
 
 
+_localdir = formencode.api.get_localedir()
+
 def set_formencode_translation(languages):
     """Set request specific translation of FormEncode."""
-    from gettext import translation
     try:
-        formencode_translation = translation('FormEncode',
-            languages=languages, localedir=formencode.api.get_localedir())
+        formencode_translation = translation(
+            'FormEncode',languages=languages, localedir=_localdir)
     except IOError, error:
         raise LanguageError('IOError: %s' % error)
     pylons.tmpl_context.formencode_translation = formencode_translation
